/***********************************************************************
    PS3 USB Host
    Copyright (C) 2013 Cross Product Creations

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************/

#include "PS3_SPI.h"

volatile BYTE spiInputBuf[SPI_INPUT_PACKET_LENGTH];

void initSPI(void)
{
    BYTE temp;
    SPI1STATbits.SISEL = 0b001; //interrupt when data is available.
    SPI1CON1 = 0;
    SPI1CON1bits.SMP = 0;
    SPI1CON1bits.SSEN = 1;
    SPI1CON1bits.MSTEN = 0; //slave mode
    SPI1CON2 = 0;
    SPI1CON2bits.SPIBEN = 1;
    switch (GetSpiBusMode())
    {
        case 0:
            UART2PrintString("SPI Bus Mode 0\n\r");
            SPI1CON1bits.CKP = 0;
            SPI1CON1bits.CKE = 1;
            break;

        case 1:
            UART2PrintString("SPI Bus Mode 1\n\r");
            SPI1CON1bits.CKP = 0;
            SPI1CON1bits.CKE = 0;
            break;

        case 2:
            UART2PrintString("SPI Bus Mode 2\n\r");
            SPI1CON1bits.CKP = 1;
            SPI1CON1bits.CKE = 1;
            break;

        case 3:
            UART2PrintString("SPI Bus Mode 3\n\r");
            SPI1CON1bits.CKP = 1;
            SPI1CON1bits.CKE = 0;
            break;

        default:
            UART2PrintString("ERROR SPI BUS MODE\n\r");
            break;
    }

    IFS0bits.SPI1IF = 0; //clear spi interrupt flag
    IPC2bits.SPI1IP = 6; ///set interrupt priority
    IEC0bits.SPI1IE = 1; //enable interrupts

    SPI1STATbits.SPIEN = 1;
    
    while(!SPI1STATbits.SPITBF)
    {
        SPI1BUF = 0xAC; //fill the buffer so we always start on byte 9
    }

    while(!SPI1STATbits.SRXMPT)
    {
        temp = SPI1BUF; //burn off bytes to empty the input buffer.
    }

}

SPI_INPUT_PACKET* getSpiInputPacket(void)
{
    return (SPI_INPUT_PACKET*)&spiInputBuf;
}
