/***********************************************************************
    PS3 USB Host
    Copyright (C) 2013 Cross Product Creations

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************/

#include "PS3_controller.h"
#include "uart2.h"
#include "PS3_SPI.h"

BYTE PS3WireInitialize[4] = {0x42, 0x0c, 0x00, 0x00}; //send this packet to start controller input data

volatile BYTE outputReportBT[58] =
{
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00,
    0xff, 0x27, 0x10, 0x00, 0x32, //enable LED4
    0xff, 0x27, 0x10, 0x00, 0x32, //enable LED3
    0xff, 0x27, 0x10, 0x00, 0x32, //enable LED2
    0xff, 0x27, 0x10, 0x00, 0x32, //enable LED1
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00
};

//USB Wire report is shorter, doesn't have the Bluetooth headers
volatile BYTE *outputReportWire = outputReportBT + 10; //careful, must add even number only since pointers are 16-bit.
volatile CONTROLLER_OUTPUT *PS3OutputPtr = (CONTROLLER_OUTPUT*) (outputReportBT+10);

//////////////////////////////PRIVATE VARIABLES//////////////////////////////////
static volatile BUTTONS LastPress;
static volatile BUTTONS NewPress;
static volatile BUTTONS PressDuration;
static volatile CONTROLLER_STATE_BUFFER CurrentStateBuffer;
static volatile ANALOGS LastAnalogs;
volatile BYTE ps3_report_buf[64];
static volatile unsigned int idleTimer = 0;

void InitHIDOutputReport(void)
{
    outputReportBT[8] = 0x52;// HID BT Set_report (0x50) | Report Type (Output 0x02)
    outputReportBT[9] = 0x01;// Report ID
    PS3OutputPtr->gyroEnable = 0xFF;
    PS3OutputPtr->gyroCalibration = 114;
}

void UpdateCurrentStateBuffer(CONTROLLER_IN *ControllerIn)
{
    unsigned int i = 0;
    for (i = 0; i<PS3_REPORT_LENGTH; i++)
    {
        ps3_report_buf[i] = ((BYTE*)ControllerIn)[i+9];
    }

        CurrentStateBuffer.Buttons.select = ControllerIn->select;
        CurrentStateBuffer.Buttons.lStickPress = ControllerIn->lStickPress;
        CurrentStateBuffer.Buttons.rStickPress = ControllerIn->rStickPress;
        CurrentStateBuffer.Buttons.start = ControllerIn->start;
        CurrentStateBuffer.Buttons.dPadUp = ControllerIn->dPadUp;
        CurrentStateBuffer.Buttons.dPadRight = ControllerIn->dPadRight;
        CurrentStateBuffer.Buttons.dPadDown = ControllerIn->dPadDown;
        CurrentStateBuffer.Buttons.dPadLeft = ControllerIn->dPadLeft;
        CurrentStateBuffer.Buttons.lTriggerPress = ControllerIn->lTriggerPress;
        CurrentStateBuffer.Buttons.rTriggerPress = ControllerIn->rTriggerPress;
        CurrentStateBuffer.Buttons.lBumper = ControllerIn->lBumper;
        CurrentStateBuffer.Buttons.rBumper = ControllerIn->rBumper;
        CurrentStateBuffer.Buttons.triangle = ControllerIn->triangle;
        CurrentStateBuffer.Buttons.circle = ControllerIn->circle;
        CurrentStateBuffer.Buttons.cross = ControllerIn->cross;
        CurrentStateBuffer.Buttons.square = ControllerIn->square;
        CurrentStateBuffer.Buttons.psButton = ControllerIn->psButton;

        CurrentStateBuffer.Analogs.lStickX = ControllerIn->lStickX;
        CurrentStateBuffer.Analogs.lStickY = ControllerIn->lStickY;
        CurrentStateBuffer.Analogs.rStickX = ControllerIn->rStickX;
        CurrentStateBuffer.Analogs.rStickY = ControllerIn->rStickY;
        CurrentStateBuffer.Analogs.lTriggerAnalog = ControllerIn->lTriggerAnalog;
        CurrentStateBuffer.Analogs.rTriggerAnalog = ControllerIn->rTriggerAnalog;
        CurrentStateBuffer.Analogs.accelX =(unsigned int)ControllerIn->accelX_0 + (((unsigned int)ControllerIn->accelX_1)<<8);
        CurrentStateBuffer.Analogs.accelY = (unsigned int)ControllerIn->accelY_0 + (((unsigned int)ControllerIn->accelY_1)<<8);
        CurrentStateBuffer.Analogs.accelZ = (unsigned int)ControllerIn->accelZ_0 + (((unsigned int)ControllerIn->accelZ_1)<<8);
        CurrentStateBuffer.Analogs.gyroZ = (unsigned int)ControllerIn->gyroZ_0 + (((unsigned int)ControllerIn->gyroZ_1)<<8);

        CurrentStateBuffer.usbCable = ControllerIn->usbCable;
        CurrentStateBuffer.battery = ControllerIn->battery;
        CurrentStateBuffer.rumbleStatus = ControllerIn->rumbleStatus;
}

void SpiInputOverride(void)
{
    if (getSpiInputPacket()->overrideSelect) CurrentStateBuffer.Buttons.select = getSpiInputPacket()->select;
    if (getSpiInputPacket()->overrideLStickPress) CurrentStateBuffer.Buttons.lStickPress = getSpiInputPacket()->lStickPress;
    if (getSpiInputPacket()->overrideRStickPress) CurrentStateBuffer.Buttons.rStickPress = getSpiInputPacket()->rStickPress;
    if (getSpiInputPacket()->overrideStart) CurrentStateBuffer.Buttons.start = getSpiInputPacket()->start;
    if (getSpiInputPacket()->overrideDPadUp) CurrentStateBuffer.Buttons.dPadUp = getSpiInputPacket()->dPadUp;
    if (getSpiInputPacket()->overrideDPadRight) CurrentStateBuffer.Buttons.dPadRight = getSpiInputPacket()->dPadRight;
    if (getSpiInputPacket()->overrideDPadDown) CurrentStateBuffer.Buttons.dPadDown = getSpiInputPacket()->dPadDown;
    if (getSpiInputPacket()->overrideDPadLeft) CurrentStateBuffer.Buttons.dPadLeft = getSpiInputPacket()->dPadLeft;
    if (getSpiInputPacket()->overrideLTriggerPress) CurrentStateBuffer.Buttons.lTriggerPress = getSpiInputPacket()->lTriggerPress;
    if (getSpiInputPacket()->overrideRTriggerPress) CurrentStateBuffer.Buttons.rTriggerPress = getSpiInputPacket()->rTriggerPress;
    if (getSpiInputPacket()->overrideLBumper) CurrentStateBuffer.Buttons.lBumper = getSpiInputPacket()->lBumper;
    if (getSpiInputPacket()->overrideRBumper) CurrentStateBuffer.Buttons.rBumper = getSpiInputPacket()->rBumper;
    if (getSpiInputPacket()->overrideTriangle) CurrentStateBuffer.Buttons.triangle = getSpiInputPacket()->triangle;
    if (getSpiInputPacket()->overrideCircle) CurrentStateBuffer.Buttons.circle = getSpiInputPacket()->circle;
    if (getSpiInputPacket()->overrideCross) CurrentStateBuffer.Buttons.cross = getSpiInputPacket()->cross;
    if (getSpiInputPacket()->overrideSquare) CurrentStateBuffer.Buttons.square = getSpiInputPacket()->square;
    if (getSpiInputPacket()->overridePsButton) CurrentStateBuffer.Buttons.psButton = getSpiInputPacket()->psButton;

    if (getSpiInputPacket()->overrideLStickX) CurrentStateBuffer.Analogs.lStickX = getSpiInputPacket()->lStickX;
    if (getSpiInputPacket()->overrideLStickY) CurrentStateBuffer.Analogs.lStickY = getSpiInputPacket()->lStickY;
    if (getSpiInputPacket()->overrideRStickX) CurrentStateBuffer.Analogs.rStickX = getSpiInputPacket()->rStickX;
    if (getSpiInputPacket()->overrideRStickY) CurrentStateBuffer.Analogs.rStickY = getSpiInputPacket()->rStickY;
    if (getSpiInputPacket()->overrideLTriggerAnalog) CurrentStateBuffer.Analogs.lTriggerAnalog = getSpiInputPacket()->lTriggerAnalog;
    if (getSpiInputPacket()->overrideRTriggerAnalog) CurrentStateBuffer.Analogs.rTriggerAnalog = getSpiInputPacket()->rTriggerAnalog;
    if (getSpiInputPacket()->overrideAccelX) CurrentStateBuffer.Analogs.accelX = (unsigned int)getSpiInputPacket()->accelX_0 + (((unsigned int)getSpiInputPacket()->accelX_1)<<8);
    if (getSpiInputPacket()->overrideAccelY) CurrentStateBuffer.Analogs.accelY = (unsigned int)getSpiInputPacket()->accelY_0 + (((unsigned int)getSpiInputPacket()->accelY_1)<<8);
    if (getSpiInputPacket()->overrideAccelZ) CurrentStateBuffer.Analogs.accelZ = (unsigned int)getSpiInputPacket()->accelZ_0 + (((unsigned int)getSpiInputPacket()->accelZ_1)<<8);
    if (getSpiInputPacket()->overrideGyroZ)  CurrentStateBuffer.Analogs.gyroZ = (unsigned int)getSpiInputPacket()->gyroZ_0 + (((unsigned int)getSpiInputPacket()->gyroZ_1)<<8);
}

/*******************************************************************************
Function:  UpdateNewPress(void)

Precondition:
    None.

Overview:
    This routine updates the states for button presses and also checks for i
    idle for the auto power-off feature.
 * PressDuration: how long the button has been held down (10ms per count)
 * NewPress: if this is a new button press

Input: level: 0-255, determines rumble strength.  6 levels available.

Output: None

*******************************************************************************/
void UpdateNewPress(void)
{
    unsigned int i;
    BOOL changeFlag = FALSE;
    unsigned int analogChange;
    
    //Update Button state record
    for (i = 0; i<17; i++)
    {
        //Set press duration counter to 1 immediately after being pressed or 0 after being released, or else it won't update until the Timer4 flag is set
        if (CurrentStateBuffer.Buttons.array[i] > 0)
        {
            PressDuration.array[i]++;
        }
        else
        {
            PressDuration.array[i] = 0;
        }

        if (CurrentStateBuffer.Buttons.array[i] > 0 && LastPress.array[i] == 0)
        {
            NewPress.array[i] = 1;
        }
        else
        {
            NewPress.array[i] = 0;
        }

        //check for changing inputs
        if (LastPress.array[i] != CurrentStateBuffer.Buttons.array[i])
        {
            changeFlag = TRUE;
        }

        LastPress.array[i] = CurrentStateBuffer.Buttons.array[i]; //update record of last cycle's button state with new state
    }

    //Update analogs state record, check for change
    for ( i = 1; i<10; i++)
    {
        if (CurrentStateBuffer.Analogs.array[i] > LastAnalogs.array[i])
        {
            analogChange = CurrentStateBuffer.Analogs.array[i] - LastAnalogs.array[i];
        }
        else
        {
            analogChange = LastAnalogs.array[i] - CurrentStateBuffer.Analogs.array[i];
        }
        if (analogChange > 5)
        {
            changeFlag = TRUE;
        }
        LastAnalogs.array[i] = CurrentStateBuffer.Analogs.array[i];
    }

    //update idle timer if no input change.
    if (changeFlag == 0)
    {
        if (idleTimer < 0xFFFF)
        {
            idleTimer ++;
        }
    }
    else
    {
        ResetIdleTimer();
    }    
}

////////////////////////GET functions////////////////////////////////////
BUTTONS* GetButtonPress(void)
{
    return (BUTTONS*)&(CurrentStateBuffer.Buttons);
}

BUTTONS* GetPressDuration(void)
{
    return (BUTTONS*)&(PressDuration);
}

BUTTONS* GetNewPress(void)
{
    return (BUTTONS*)&(NewPress);
}

ANALOGS* GetAnalogs(void)
{
    return (ANALOGS*)&(CurrentStateBuffer.Analogs);
}

//Returns PS3 input report.  We first byte to 0xFF to identify stagnant data.
BYTE* GetPS3Report(void)
{
    return (BYTE*)ps3_report_buf;
}

CONTROLLER_OUTPUT* GetOutputPacket(void)
{
    return (CONTROLLER_OUTPUT*)PS3OutputPtr;
}

void SetLEDBin(unsigned char value)
{
    PS3OutputPtr->LED = value<<1;
}

void SetLED1(BOOL value)
{
    if (value == 1) PS3OutputPtr->LED |= 0x02;
    else if (value == 0) PS3OutputPtr->LED &= ~0x02;
}

void SetLED2(BOOL value)
{
    if (value == 1) PS3OutputPtr->LED |= 0x04;
    else if (value == 0) PS3OutputPtr->LED &= ~0x04;
}

void SetLED3(BOOL value)
{
    if (value == 1) PS3OutputPtr->LED |= 0x08;
    else if (value == 0) PS3OutputPtr->LED &= ~0x08;
}

void SetLED4(BOOL value)
{
    if (value == 1) PS3OutputPtr->LED |= 0x10;
    else if (value == 0) PS3OutputPtr->LED &= ~0x10;
}

/*******************************************************************************
Function: SetRumbleH(unsigned char level)

Precondition:
    None.

Overview:
    This turns on the fast rumble

Input: level: 0-255, determines rumble strength.  6 levels available.

Output: None

*******************************************************************************/
void SetRumbleH(unsigned char level)
{
    unsigned char duration;
    duration = level/43; //translate 0-255 to 0-5;
    /* Level 0: 0-42
     * Level 1: 43-85
     * Level 2: 86-128
     * Level 3: 129-171
     * Level 4: 172-214
     * Level 5: 215-255
     */

    if (duration == 0){
        PS3OutputPtr->rumbleHOn = 0;
        PS3OutputPtr->rumbleHDuration = 0;
    }
    else {
        PS3OutputPtr->rumbleHOn = 1;
        PS3OutputPtr->rumbleHDuration = duration;
    }
}

/*******************************************************************************
Function: SetRumbleL(unsigned char level)

Precondition:
    None.

Overview:
    This turns on the fast rumble

Input: level: 0-255, determines rumble strength.

Output: None

*******************************************************************************/
void SetRumbleL(unsigned char level)
{
    PS3OutputPtr->rumbleLPower = level;
    PS3OutputPtr->rumbleLDuration = 0x32;
}


void UpdateBatteryDisplay(CONTROLLER_IN* Controller){
    static unsigned int displayBattCounter;

    //override LED state with battery meter if PS button is pressed.
    if (Controller->psButton == 1)
    {
        displayBattCounter = 10;
    }
    if (displayBattCounter > 0)
    {
        switch (Controller->battery)
        {
            case 0x05:
                SetLEDBin(0x0F);
                break;
            case 0x04:
                SetLEDBin(0x07);
                break;
            case 0x03:
                SetLEDBin(0x03);
                break;
            case 0x02:
                SetLEDBin(0x01);
                break;
            default:
                SetLEDBin(0);
        }
        displayBattCounter--;
        if (displayBattCounter == 0)
        {
            SetLEDBin(0);
        }
    }
}

//This function is used to control timing for rumble feedback when setting configurations.
void UpdateRumbleFeedback(unsigned int duration)
{
    static unsigned int rumbleCounter;

    // if duration > 0, then it is a set operation
    //if duration == 0, then decrement the counter
    if (duration > 0) //set duration
    {
        rumbleCounter = duration;
        SetRumbleH(0xFF);
    }
    else if (duration == 0) //update timer
    {
        if ( rumbleCounter > 0)
        {
            SetRumbleH(0xFF);
            rumbleCounter--;
            if (rumbleCounter == 0){
                SetRumbleH(0);
            }
        }
    }
}

//Check idle timer used for auto-disconnect
unsigned int GetIdleTimer(void)
{
    return idleTimer;
}

//reset idle timer used for auto-disconnect
void ResetIdleTimer(void)
{
    idleTimer = 0;
}



