//#define PRINT_ACL_PACKET
//#define PRINT_HCI_PACKET
//#define DEBUG_MODE

/******************************************************************************
            Acknowledgements

 * This project was  possible thanks to previous work from many people:
 * 
 * The Microchip PIC USB Bluetooth dongle driver framework was based off of work by
 * "yts" on the Microchip forums.
 *  http://www.microchip.com/forums/m489233.aspx
 *
 * A lot of Bluetooth and Sony DualShock driver work was done by the following:
 * Oleg
 * http://www.circuitsathome.com/mcu/ps3-and-wiimote-game-controllers-on-the-arduino-host-shield-part-1
 *
 * Kristian Lauszus 
 * http://blog.tkjelectronics.dk/2011/09/fez-panda-ps3-controller-via-bluetooth/
 *
 * Guillem Vials Gangolells
 * http://www.guillem.co.uk/projects
 *
 * "Acxie"
 * http://www.motioninjoy.com/
 *
 * The bootloader is Mikael Gustafsson's excellent ds30 bootloader.
 * http://mrmackey.no-ip.org/elektronik/ds30loader/
 *
 * Lots of protocol decoding was done with WireShark:
 * http://www.wireshark.org/
******************************************************************************/

/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the CompanyE for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/
/***********************************************************************
    PS3 USB Host
    Copyright (C) 2013 Cross Product Creations
    The GNU Public License applies to code NOT from Microchip.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************/

/*********************TODO LIST******************************************
 * find a way to flush the SPI output buffer instead of filling it up so we know where the first byte is
 *
 * *********************************************************************/


#include "main.h"
#include <p24Fxxxx.h>
#include <stdlib.h>

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_generic_PS3.h"
#include "DEE Emulation 16-bit/DEE Emulation 16-bit.h"

#include "PS3_controller.h"
#include "outputs.h"
#include "PS3_SPI.h"

//firmware revision


/////////////////Library configuration//////////////
#define USE_AND_OR
// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************
#define STOP_TIMER_IN_IDLE_MODE     0x2000
#define TIMER_SOURCE_INTERNAL       0x0000
#define TIMER_ON                    0x8000
#define GATED_TIME_DISABLED         0x0000
#define TIMER_16BIT_MODE            0x0000

#define TIMER_PRESCALER_1           0x0000
#define TIMER_PRESCALER_8           0x0010
#define TIMER_PRESCALER_64          0x0020
#define TIMER_PRESCALER_256         0x0030
#define TIMER_INTERRUPT_PRIORITY    0x0001

	_CONFIG2(IESO_OFF & PLL_96MHZ_ON & PLLDIV_DIV5 & FNOSC_PRIPLL & POSCMOD_HS)   // Primary HS OSC with PLL, USBPLL /5
	_CONFIG1(JTAGEN_OFF & ICS_PGx2 & FWDTEN_OFF)        // JTAG off, watchdog timer off
 
// *****************************************************************************


// Application States

typedef enum
{
    BT_INITIALIZE = 0,                // Initialize the app when a device is attached
    BT_STATE_PROCESS,
    BT_STATE_READ_DATA
} BT_STATE;


// Hci States
typedef enum
{
	HCI_CMD_RESET = 0,                // Initialize the hci when a device is attached
                //HCI_CMD_RESET_WAIT,
	HCI_CMD_READ_BD_ADDR,
                //HCI_CMD_READ_BD_ADDR_WAIT,
	HCI_CMD_LOCAL_NAME,
                //HCI_CMD_LOCAL_NAME_WAIT,
	HCI_CMD_CLASS_DEVICE,
                //HCI_CMD_CLASS_DEVICE_WAIT,
	HCI_CMD_SCAN_ENABLE,
        HCI_CMD_SCAN_ENABLE_WAIT,
        HCI_CMD_INCOMING_ACCEPT,
        HCI_ROLE_CHANGE_WAIT,
	HCI_CMD_CONNECTION_ACCEPTED,
        HCI_CMD_SCAN_DISABLE,

	L2CAP_CONNECT_RESP11,
	L2CAP_CONFIG_REQ11,
	L2CAP_CONFIG_RESP11,
	L2CAP_CONNECT_RESP13,
	L2CAP_CONFIG_REQ13,
	L2CAP_CONFIG_RESP13,
                
        PS3_BT_SET_FEATURE_INIT,
        PS3_BT_RUNNING,

        L2CAP_DISCONNECT_DATA_RESP,
        L2CAP_DISCONNECT_CTRL_RESP,
        L2CAP_DISCONNECT_DATA,
        L2CAP_DISCONNECT_CTRL,
        HCI_DISCONNECT,

        HCI_READ,

} HCI_STATE;

typedef enum {
    PS3_WIRE_WRITE_BD_ADDR,
    PS3_WIRE_READ_BD_ADDR,
    PS3_WIRE_CONTROLLER_INIT,
    PS3_WIRE_RUNNING
} PS3_WIRE_STATE;

// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE	deviceAddress;  // Address of the device on the USB
BT_STATE  DemoState;      // Current state of the demo application
HCI_STATE  HciState;      // Current state of the demo application
PS3_WIRE_STATE PS3WireState; //state machine when usb controller is attached.

int data_num;
union __PACKED{
    unsigned char bytes[6];//
    unsigned int ints[3];
} localBdAddr;

BYTE remote_bd_addr[6];//
BYTE acl_buf[DATA_PACKET_LENGTH];
BYTE hci_buf[DATA_PACKET_LENGTH];

BYTE handle[2];//a handle for ACL
BYTE cid_dev_ctrl[2];
BYTE cid_host_ctrl[2] = {0x40, 0x00};//assign this channel id
BYTE cid_dev_data[2];
BYTE cid_host_data[2] = {0x41, 0x00};
BYTE packet_id;

char message[64];
static volatile int spi_index = 0;

//variables to access PS3 packets
CONTROLLER_IN *ControllerInPtr = (CONTROLLER_IN*)acl_buf;
extern BYTE PS3WireInitialize[4];
extern BYTE outputReportBT[58];
extern BYTE *outputReportWire;

unsigned int i; //general purpose index counter
static volatile unsigned int loopCounter; //counts the numerber of times the loop is executed, used for scheduling output packets
static volatile unsigned int timeoutCounter = 0; //counter for idle disconnect
static volatile BOOL configFlag = 0; //TRUE if we're in configuration mode

unsigned char vibeL;
unsigned char vibeH;
unsigned char LEDVal = 1;

char temp; //general purpose temp char
//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        TestCases
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    None
 *
 * Overview:        This routine is used for use with the automatic testbed.
 *                  clocking the PGC pin will toggle the outputs on and off
 *                  one by one.
 *************************************************************************/
void TestCases(void)
{
    static unsigned int pinNum = 0;
    static BOOL lastClockState = 0;
    BOOL clockState;

    clockState = _PGC;

    if (clockState == 1 && lastClockState == 0)
    {
        UART2PrintString("Test Case: ");
        UART2PutDecInt(pinNum);
        UART2PrintString("\n\r");

        //Disable Pull-up on RD11 Slave Select;
        CNPU4bits.CN56PUE = 0;

        //Set all pins to inputs in case of shorting
        TRISB = 0xFFFF;
        TRISC = 0xFFFF;
        TRISD = 0xFFFF;
        TRISE = 0xFFFF;
        TRISF = 0xFFFF;
        TRISG = 0xFFFF;

        LATB = 0;
        LATC = 0;
        LATD = 0;
        LATE = 0;
        LATF = 0;
        LATG = 0;

        switch (pinNum)
        {
            case 0: //PWM0
                _TRISG6 = 0;
                _LATG6 = 1;
                break;
            case 1: //PWM1
                _TRISG7 = 0;
                _LATG7 = 1;
                break;
            case 2: //PWM2
                _TRISG8 = 0;
                _LATG8 = 1;
                break;
            case 3: //PWM3
                _TRISG9 = 0;
                _LATG9 = 1;
                break;
            case 4: //PWM4
                _TRISB5 = 0;
                _LATB5 = 1;
                break;
            case 5: //PWM5
                _TRISB4 = 0;
                _LATB4 = 1;
                break;
            case 6: //PWM6
                _TRISB2 = 0;
                _LATB2 = 1;
                break;
            case 7: //PWM7
                _TRISB1 = 0;
                _LATB1 = 1;
                break;
            case 8: //PWM8
                _TRISB0 = 0;
                _LATB0 = 1;
                break;
            case 9: //PWM9
                _TRISB8 = 0;
                _LATB8 = 1;
                break;
            case 10: //RumbleL
                _TRISB10 = 0;
                _LATB10 = 1;
                break;
            case 11: //RumbleH
                _TRISB11 = 0;
                _LATB11 = 1;
                break;
            case 12: //LED0
                _TRISB12 = 0;
                _LATB12 = 1;
                break;
            case 13: //LED1
                _TRISB13 = 0;
                _LATB13 = 1;
                break;
            case 14: //B0
                _TRISF3 = 0;
                _LATF3 = 1;
                break;
            case 15: //SCKIN
                _TRISD8 = 0;
                _LATD8 = 1;
                break;
            case 16: //SDI1
                _TRISD9 = 0;
                _LATD9 = 1;
                break;
            case 17: //SDO1
                _TRISD10 = 0;
                _LATD10 = 1;
                break;
            case 18: //SSIN
                _TRISD11 = 0;
                _LATD11 = 1;
                break;
            case 19: //B1
                _TRISC13 = 0;
                _LATC13 = 1;
                break;
            case 20: //B2
                _TRISC14 = 0;
                _LATC14 = 1;
                break;
            case 21: //B3
                _TRISD1 = 0;
                _LATD1 = 1;
                break;
            case 22: //B4
                _TRISD2 = 0;
                _LATD2 = 1;
                break;
            case 23: //B5
                _TRISD3 = 0;
                _LATD3 = 1;
                break;
            case 24: //B6
                _TRISD4 = 0;
                _LATD4 = 1;
                break;
            case 25: //B7
                _TRISD5 = 0;
                _LATD5 = 1;
                break;
            case 26: //B8
                _TRISD6 = 0;
                _LATD6 = 1;
                break;
            case 27: //B9
                _TRISD7 = 0;
                _LATD7 = 1;
                break;
            case 28: //B10
                _TRISF0 = 0;
                _LATF0 = 1;
                break;
            case 29: //B11
                _TRISF1 = 0;
                _LATF1 = 1;
                break;
            case 30: //B12
                _TRISE0 = 0;
                _LATE0 = 1;
                break;
            case 31: //B13
                _TRISE1 = 0;
                _LATE1 = 1;
                break;
            case 32: //B14
                _TRISE2 = 0;
                _LATE2 = 1;
                break;
            case 33: //B15
                _TRISE3 = 0;
                _LATE3 = 1;
                break;
            case 34: //B16
                _TRISE4 = 0;
                _LATE4 = 1;
                break;
            default:
                break;
        }
        pinNum ++;
    }
    lastClockState = clockState;
}
/*************************************************************************
 * Function:        ResetStates
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    None
 *
 * Overview:        This routine will reset the internal states to the
 *                  power-on state.
 *************************************************************************/
void ResetStates(void)
{
    UART2PrintString("***Reset States.***\n\r");
    DemoState = BT_INITIALIZE;
    HciState = HCI_CMD_RESET;
    PS3WireState = PS3_WIRE_WRITE_BD_ADDR;
    ResetOutputs(ControllerInPtr);
}

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/
BOOL InitializeSystem ( void )
{
        OSCCON = 0b1111001100000000;    // Enable secondary oscillator
        CLKDIV = 0x0000;    // Set PLL prescaler (1:1)

//Initialize Output pins to 0 to prevent floating outputs:
        PORTB = 0;
        PORTC = 0;
        PORTD = 0;
        PORTE = 0;
        PORTF = 0;
        PORTG = 0;

        TRISB = 0b1111110011001000;
        TRISC = 0b1001111111111111;
        TRISD = 0b1111111100000001;
        TRISE = 0b1111111111000000;
        TRISF = 0b1111111111110100;
        TRISG = 0b1111110000111111;
        
//Make PGC and PGD inputs
        _TRISB6 = 1; //PGC
        _TRISB7 = 1; //PGD
////////////////Map SPI Bus///////////////////////////////

        RPINR20bits.SCK1R = 2; //map SCK1IN to RP2
        RPINR20bits.SDI1R = 4; //map SDI1 to RP4
        RPINR21bits.SS1R = 12; //map SS1R to RP12
        RPOR1bits.RP3R = 7; //map RP3 to SDO1

        //Map interrupt for /SS line
        _TRISD0 = 1;
        RPINR0bits.INT1R = 11;
        IPC5bits.INT1IP = 5; //set slave select interrupt priority
        INTCON2bits.INT1EP = 0; //interrupt on positive edge
        IFS1bits.INT1IF = 0;

////////////////Set internal pull-up/pull-down resistors//////////////
        //Pull down programming lines
        CNPD2bits.CN24PDE = 1;
        CNPD2bits.CN25PDE = 1;

        //Pull down LED0, LED1
        CNPD2bits.CN30PDE = 1;
        CNPD2bits.CN31PDE = 1;

        //Pull up Slave Select line
        CNPU4bits.CN56PUE = 1;


/////////////Use Timer4 for output poll period
        T4CON = 0; //clear state
        T4CONbits.TSIDL = 0; //Continue in Idle
        T4CONbits.TGATE = 0; //No gate accumulation
        T4CONbits.TCKPS = 0b10; //Prescale 1:64
        T4CONbits.T32 = 0; //16-bit mode
        T4CONbits.TCS = 0; //Clock Source: Internal Clock (Fosc/2 = 16MHz)
        TMR4 = 0;
        PR4 = 2500; //2500 with presacel 64 is 10ms
        T4CONbits.TON = 1;
//////////////////////////////////

  // Configure UART2 Functions
	//Assign U2RX To Pin RP10,RF4
	RPINR19bits.U2RXR = 10;
	//Assign U2TX To Pin RP17,RF5
	RPOR8bits.RP17R = 5;

    UART2Init();
    UART2PrintString("UART Initialized/.\n\r");

///////////////SETUP ADC////////////////////
    AD1PCFGL = 0b1111001111111111; // Configure AN10 and AN11 as analog
    AD1CSSL = ~AD1PCFGL; // Include AN10 and AN11 in scan
    AD1CON1bits.FORM = 0b00; // Fractional
    AD1CON1bits.SSRC = 0b111; //Internal counter triggers conversion
    AD1CON1bits.ASAM = 0; //manual sampling start
    AD1CON1bits.SAMP = 1;


    AD1CON2bits.CSCNA = 1; //scan channels in AD1CSSL
    AD1CON2bits.VCFG = 0; //Use AVdd and AVss as references
    AD1CON2bits.CSCNA = 1; //scan inputs from AD1CSSL
    AD1CON2bits.SMPI = 0b0001; //interrupts after 2nd sample
    AD1CON2bits.BUFM = 0; //16bit buffer
    AD1CON2bits.ALTS = 0; //only use MUX A

    AD1CON3 = 0b0001111100111111; // Sample time = 15Tad, Tad = Tcy
    AD1CON1bits.ADON = 1; // turn ADC ON


    // Set Default demo state
    UART2PrintString("Reset States on Init.\n\r");
    ResetStates();
    return TRUE;
} // InitializeSystem

/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
            UART2PrintString( "Generic demo device attached - polled, deviceAddress=" );
            UART2PutDec( deviceAddress );
            UART2PrintString( "\r\n" );
            return TRUE;
        }
    }

    return FALSE;

} // CheckForNewAttach


//Parse HCI packets
void ParseRx1PS3BT(void){
    unsigned int opCode;

#ifdef PRINT_HCI_PACKET
    unsigned int data_num;
    UART2PrintString("HCI_RX: ");
    for (data_num = 0; data_num<USBHostGenericGetRx1Length(deviceAddress); data_num++)
    {
        UART2PutHex(hci_buf[data_num]);
        UART2PutChar(' ');
    }
    UART2PrintString("-- ");
#endif
    DemoState = BT_STATE_PROCESS;//send program back to BT_STATE_PROCESS by default
    opCode = ((unsigned int)hci_buf[3])*256 + hci_buf[4];
    switch(hci_buf[0])
    { //Switch by event code
        case 0x0E: //command complete event
            DemoState = BT_STATE_PROCESS;
            UART2PrintString("CMD complete: ");
            switch (opCode)
            {
                case 0x030c:  //if (hci_buf[3] == 0x03 && hci_buf[4]==0x0c) //Reset complete code.  Assumes success/.
                    UART2PrintString("Reset.\n\r");
                    HciState = HCI_CMD_READ_BD_ADDR;
                    break;

                case 0x0910:
                    UART2PrintString("Read Address. Local BD_ADDR: ");
                    //save address
                    for (data_num = 0; data_num < 6; data_num++)
                    {
                        localBdAddr.bytes[data_num] = hci_buf[5-data_num+6];
                        UART2PutHex(localBdAddr.bytes[data_num]);
                        UART2PutChar(' ');
                    }
                    UART2PrintString(". Storing address.\n\r");
                    DataEEWrite(localBdAddr.ints[0], EEPROM_BT_ADDR_0);
                    DataEEWrite(localBdAddr.ints[1], EEPROM_BT_ADDR_1);
                    DataEEWrite(localBdAddr.ints[2], EEPROM_BT_ADDR_2);
                    UART2PutHexWord(DataEERead(EEPROM_BT_ADDR_0));
                    UART2PutHexWord(DataEERead(EEPROM_BT_ADDR_1));
                    UART2PutHexWord(DataEERead(EEPROM_BT_ADDR_2));
                    UART2PrintString("\n\r");
                    
                    break;

                case 0x130c:
                    UART2PrintString("Local Name.\n\r");
                    break;

                case 0x240c:
                    UART2PrintString("Write Class.\n\r");
                    break;

                case 0x1a0c:
                    UART2PrintString("Write Scan Enable/Disable.\n\r");
                    break;

                default:
                    UART2PrintString("Other.\n\r");
                    break;
            }
            break;

        case 0x12: //Event: Role Change Status
            if (hci_buf[2] == 0x00)
            {
                UART2PrintString("Role Change Successful.\n\r");
                HciState = HCI_CMD_CONNECTION_ACCEPTED;
            }
            else
            {
                UART2PrintString("Role Change Failed.  Error: ");
                UART2PutHex(hci_buf[2]);
                UART2PrintString("\n\r");
                HciState = HCI_CMD_RESET;
            }		
            break;

        case 0x03: //connection complete
            UART2PrintString("Connection Accepted Event: ");
            if (hci_buf[2] == 0)
            {
                handle[0] = hci_buf[3];
                handle[1] = hci_buf[4];//save connection handle and add (PB flag + BC flag)
                UART2PrintString("Success. Handle = ");
                UART2PutHex(hci_buf[4]);
                UART2PutHex(hci_buf[3]);
                HciState = HCI_READ; //wait for incoming request from controller
            }
            else
            {
                UART2PrintString("Error: ");
                UART2PutHex(hci_buf[2]);
                HciState = HCI_CMD_RESET;
            }
            UART2PrintString("\n\r");
            break;

        case 0x04:  //incoming connection attempt
            if (HciState == HCI_CMD_SCAN_ENABLE_WAIT)
            {
                UART2PrintString("Incoming Connection: Remote_Bd_Addr = ");
                remote_bd_addr[0] = hci_buf[2];
                remote_bd_addr[1] = hci_buf[3];
                remote_bd_addr[2] = hci_buf[4];
                remote_bd_addr[3] = hci_buf[5];
                remote_bd_addr[4] = hci_buf[6];
                remote_bd_addr[5] = hci_buf[7];

                for (data_num = 0; data_num < 6; data_num++)
                {
                    UART2PutHex(remote_bd_addr[data_num]);
                    UART2PutChar(' ');
                }
                UART2PrintString("\n\r");
                HciState = HCI_CMD_INCOMING_ACCEPT;
            }
            break;

        case 0x0F:
            UART2PrintString("Command Status Event: ");
            switch (hci_buf[2])
            {
                case 0x00:
                    UART2PrintString("Pending.");
                    DemoState = BT_STATE_READ_DATA;
                    break;
                    
                case 0x02:
                    UART2PrintString("Error 0x02: Unknown Connection ID.");
                    HciState = HCI_CMD_RESET;
                    break;

                default:
                    UART2PrintString("Error Code:");
                    UART2PutHex(hci_buf[2]);
                    HciState = HCI_CMD_RESET;
                    break;
            }

            UART2PrintString("\n\r");
            break;

        case 0x05:
            UART2PrintString("Disconnect Event: ");
            if (hci_buf[2] == 0)
            {
                UART2PrintString("Success.");
                HciState = HCI_CMD_RESET;
            }
            else
            {
                UART2PrintString("Error.");
            }
            UART2PrintString("\n\r");
            ResetStates();
            break;

        case 0x13:
            #ifdef PRINT_HCI_PACKET
            UART2PrintString("Completed Packets.\n\r");
            DemoState = BT_STATE_READ_DATA;
            #endif
            break;
        default:
            #ifdef PRINT_HCI_PACKET
            UART2PrintString("Unknown.\n\r");
            #endif

            DemoState = BT_STATE_READ_DATA;
            break;
    }

}


//Parese ACL Packet
void ParseRx2PS3BT(void){
    #ifdef PRINT_ACL_PACKET
        UART2PrintString("ACL_RX: ");
        for (data_num = 0; data_num<USBHostGenericGetRx2Length(deviceAddress); data_num++)
        {
            UART2PutHex(acl_buf[data_num]);
            UART2PutChar(' ');
        }
        UART2PrintString("-- ");
    #endif
    
    DemoState = BT_STATE_PROCESS;//send program back to BT_STATE_PROCESS
    if (acl_buf[6]==0x01 && acl_buf[7] == 0x00)
    {
        packet_id = acl_buf[9];
        UART2PrintString("CID=0x0001. ");
        switch(acl_buf[8])
        {
#ifndef PRINT_ACL_PACKET
            for (data_num = 0; data_num<USBHostGenericGetRx2Length(deviceAddress); data_num++)
            {
                UART2PutHex(acl_buf[data_num]);
                UART2PutChar(' ');
            }
#endif
            case 0x02:
                UART2PrintString("Code=0x02.(Connection Req).");
                if (acl_buf[12]==0x11 && acl_buf[13]==0x00)
                {
                    UART2PrintString("PSM=0x0011. SCID=");
                    UART2PutHex(acl_buf[15]);
                    UART2PutHex(acl_buf[14]);
                    UART2PrintString("\n\r");
                    cid_dev_ctrl[0] = acl_buf[14];
                    cid_dev_ctrl[1] = acl_buf[15];
                    HciState = L2CAP_CONNECT_RESP11;
                }
                else if (acl_buf[12]==0x13 && acl_buf[13]==0x00)
                {
                    UART2PrintString("PSM=0x0013. SCID=");
                    UART2PutHex(acl_buf[15]);
                    UART2PutHex(acl_buf[14]);
                    UART2PrintString("\n\r");
                    cid_dev_data[0] = acl_buf[14];
                    cid_dev_data[1] = acl_buf[15];
                    HciState = L2CAP_CONNECT_RESP13;
                }
                else{
                    UART2PrintString("!!!Error: PSM not supported.\n\r");
                }
                break;

            case 0x04:
                UART2PrintString("Code=0x04(Config Req).\n\r");
                if (acl_buf[12]==cid_host_ctrl[0] && acl_buf[13]==cid_host_ctrl[1])
                {
                    UART2PrintString("HID Control Channel.\n\r");
                    HciState = L2CAP_CONFIG_RESP11;
                }
                else if (acl_buf[12]==cid_host_data[0] && acl_buf[13]==cid_host_data[1])
                {
                    UART2PrintString("HID Data Channel.\n\r");
                    HciState = L2CAP_CONFIG_RESP13;
                }
                else
                {
                    UART2PrintString("!!!Error: Unknown Channel ID.\n\r");
                }
                break;

            case 0x05:
                UART2PrintString("Code=0x05(Config Resp).\n\r");
                break;

            case 0x06:
                UART2PrintString("Disconnect request:");
                if (acl_buf[12] == cid_host_data[0] && acl_buf[13] == cid_host_data[1])
                {
                    UART2PutHex(cid_host_data[0]);
                    UART2PrintString(" DATA.\n\r");
                    HciState = L2CAP_DISCONNECT_DATA_RESP;
                }
                else if (acl_buf[12] == cid_host_ctrl[0] && acl_buf[13] == cid_host_ctrl[1])
                {
                    UART2PutHex(cid_host_ctrl[0]);
                    UART2PrintString(" CTRL.\n\r");
                    HciState = L2CAP_DISCONNECT_CTRL_RESP;
                }

            case 0x07:
                UART2PrintString("Disconnect Response:");
                if (acl_buf[12] == cid_dev_data[0] && acl_buf[13] == cid_dev_data[1])
                {
                    UART2PutHex(cid_dev_data[0]);
                    UART2PrintString(" DATA.\n\r");
                    HciState = L2CAP_DISCONNECT_CTRL;
                }
                else if (acl_buf[12] == cid_dev_ctrl[0] && acl_buf[13] == cid_dev_ctrl[1])
                {
                    UART2PutHex(cid_dev_ctrl[0]);
                    UART2PrintString(" CTRL.\n\r");
                    HciState = HCI_DISCONNECT;
                }

            default:
                UART2PrintString("\n\r");
                break;

         }
    }
    else if (acl_buf[6] == cid_host_data[0] && acl_buf[7] == cid_host_data[1])
    {
        switch(acl_buf[8]){
            case 0xa1:  //Data from controller
                if (acl_buf[9] == 0x01) //Input Report
                {
                    if (ControllerInPtr->empty == 0)
                    {
                        UpdateCurrentStateBuffer(ControllerInPtr);
                        #ifdef PRINT_ACL_PACKET
                        UART2PrintString("Input Report.\n\r");
                        #endif
                    }
                    else {
                        #ifdef PRINT_ACL_PACKET
                        UART2PrintString("Empty Report.\n\r");
                        #endif
                    }
                }
                break;
            default:
                UART2PrintString("\n\r");
                break;
        }
    }
    else {
        #ifdef PRINT_ACL_PACKET
        UART2PrintString("\n\r");
        #endif
    }
}

void ParseRx1PS3Wire(void)
{
    #ifdef PRINT_HCI_PACKET
    unsigned int data_num;
    UART2PrintString("HCI_RX: ");
    for (data_num = 0; data_num<USBHostGenericGetRx1Length(deviceAddress); data_num++)
    {
        UART2PutHex(hci_buf[data_num]);
        UART2PutChar(' ');
    }
    UART2PrintString("-- ");
    #endif
    if (hci_buf[0] == 0x01)
    {
        #ifdef PRINT_HCI_PACKET
        UART2PrintString("Input Report.");
        #endif
        for (data_num = 0; data_num<USBHostGenericGetRx1Length(deviceAddress); data_num++)
        {
            acl_buf[data_num+9] = hci_buf[data_num]; 
        }
        UpdateCurrentStateBuffer(ControllerInPtr);
    }

    #ifdef PRINT_HCI_PACKET
    UART2PrintString("\n\r");
    #endif
}

/*Write functions for USB*/
BYTE Write_HCI_Command(char *message, BYTE *hci_buf, unsigned int data_size1)
{
    BYTE returnVal;

    returnVal = USBHostGenericClassRequest( deviceAddress, hci_buf, data_size1 );
    if ( returnVal == USB_SUCCESS )
    {
        #ifdef DEBUG_MODE
            unsigned int data_num;
            UART2PrintString( message);
            for(data_num=0;data_num<data_size1;data_num++)
            {
                UART2PutHex(hci_buf[data_num]);UART2PutChar(' ');
            }
                    UART2PrintString( "\r\n" );
	#endif
	//UART2PrintString( "HCI COMMAND SENT\r\n" );
    }
    else
    {
        UART2PrintString( "Write Class Error: " );
        UART2PutHex(returnVal);
        UART2PrintString("\n\r");
    }
    DelayMs(1);
    return returnVal;
}

BYTE Write_ACL_Command(char *message, BYTE *acl_buf, int data_size2)
{
    BYTE retVal = USB_BUSY;
    retVal = USBHostGenericAclWrite( deviceAddress, acl_buf, data_size2 );
    if (retVal == USB_SUCCESS)
    {
	#ifdef DEBUG_MODE
        unsigned int data_num;
        UART2PrintString( message);
        for(data_num=0;data_num<data_size2;data_num++)
        {
            UART2PutHex(acl_buf[data_num]);UART2PutChar(' ');
        }
        UART2PrintString( "\r\n" );
	#endif
       // UART2PrintString( "ACL_DATA_SENT!\n\r" );
    }
    else
    {
        UART2PutHex(retVal);
        UART2PrintString( " - ACL cannot be written ***********************");
        UART2PrintString("DemoState:");
        UART2PutDec(DemoState);
        UART2PrintString("HciState:");
        UART2PutDec(HciState);
        UART2PrintString( "\r\n" );
        for(data_num=0;data_num<data_size2;data_num++)
        {
            UART2PutHex(acl_buf[data_num]);UART2PutChar(' ');
        }
        UART2PrintString("end debug\n\r");
    }
    DelayMs(1);
    return retVal;
}

BYTE WriteOutputReport(void)
{
    BYTE retVal;
    if (GetConnectedDeviceType() == BLUETOOTH)
    {
        retVal = Write_ACL_Command(message, outputReportBT, 58);
    }
    else if (GetConnectedDeviceType() == CONTROLLER)
    {
        retVal = USBHostIssueDeviceRequest( deviceAddress, 0x21, 0x09, 0x0201, 0, 48, outputReportWire, USB_DEVICE_REQUEST_SET, 1);
    }
    else
    {
        UART2PrintString("Error: Cannot find correct type.\n\r");
        retVal = USB_ILLEGAL_REQUEST;
    }
    return retVal;
}


/*************************************************************************
 * Function:        ManageDemoStateBluetooth
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoStateBluetooth ( void )
{
/*
    UART2PutHex(DemoState);
    UART2PutChar('-');
    UART2PutHex(HciState);
    UART2PutChar(' ');
*/
    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
	#ifdef DEBUG_MODE
        UART2PrintString( "Bluetooth adapter detached - polled\r\n" );
	#endif
        ResetStates();
        SetConnectedDeviceType(NONE);
    }

    switch (DemoState)
    {
    case BT_INITIALIZE:
        UART2PrintString("INIT\n\r");
        DemoState = BT_STATE_PROCESS;
        HciState = HCI_CMD_RESET; //MH
        break;


    case BT_STATE_PROCESS:
        switch (HciState)
        {
//HCI layer***********************************************************************
            case HCI_CMD_RESET:
                timeoutCounter = 0;
                ResetOutputs(ControllerInPtr);
                DelayMs(100);  //pause a bit to let whatever transactions were going on finish.
                hci_buf[0]=0x03;
                hci_buf[1]=0x0c;
                hci_buf[2]=0;

                strcpy(message,"HCI_CMD_RESET: ");//message for BT_STATE_READ_HCI
                if (Write_HCI_Command(message, hci_buf,3) == USB_SUCCESS)
                {
                    HciState = HCI_CMD_READ_BD_ADDR;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

            case HCI_CMD_READ_BD_ADDR:
                hci_buf[0]=0x09;
                hci_buf[1]=0x10;
                hci_buf[2]=0;

                strcpy(message,"HCI_CMD_READ_BD_ADDR: ");
                if (Write_HCI_Command(message, hci_buf,3) == USB_SUCCESS)
                {
                    HciState = HCI_CMD_LOCAL_NAME;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

//********************************************************************************
            case HCI_CMD_LOCAL_NAME:
                hci_buf[0]=0x13;
                hci_buf[1]=0x0c;
                hci_buf[2]=0x04;
                hci_buf[3]='C';
                hci_buf[4]='P';
                hci_buf[5]='C';
                hci_buf[6]=0x00;

                strcpy(message,"HCI_CMD_LOCAL_NAME: ");
                if (Write_HCI_Command(message, hci_buf,7) == USB_SUCCESS)
                {
                    HciState = HCI_CMD_CLASS_DEVICE;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

//********************************************************************************
            case HCI_CMD_CLASS_DEVICE:
                hci_buf[0]=0x24;
                hci_buf[1]=0x0c;
                hci_buf[2]=0x03;
                hci_buf[3]=0x08;//gamepad
                hci_buf[4]=0x05;//joystick
                hci_buf[5]=0x00;

                strcpy(message,"HCI_CMD_CLASS_DEVICE: ");
                if (Write_HCI_Command(message, hci_buf,6) == USB_SUCCESS)
                {
                    HciState = HCI_CMD_SCAN_ENABLE;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

            case HCI_CMD_SCAN_ENABLE:
                hci_buf[0]=0x1a;
                hci_buf[1]=0x0c;
                hci_buf[2]=0x01;
                hci_buf[3]=0x03; //enable page

                strcpy(message,"HCI_CMD_SCAN_ENABLE: ");
                if (Write_HCI_Command(message, hci_buf,4) == USB_SUCCESS)
                {
                    HciState = HCI_CMD_SCAN_ENABLE_WAIT;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

            case HCI_CMD_SCAN_ENABLE_WAIT:
                 USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
                break;

            case HCI_CMD_INCOMING_ACCEPT: //Accept connection, role change to master
                hci_buf[0]=0x09;
                hci_buf[1]=0x04;
                hci_buf[2]=0x07;
                hci_buf[3]=remote_bd_addr[0];//******************************************
                hci_buf[4]=remote_bd_addr[1];//BD address (6 octets) of the slave bluetooth
                hci_buf[5]=remote_bd_addr[2];//
                hci_buf[6]=remote_bd_addr[3];//
                hci_buf[7]=remote_bd_addr[4];//
                hci_buf[8]=remote_bd_addr[5];//******************************************
                hci_buf[9]=0x00;

                strcpy(message,"HCI_CMD_ACCEPT_INCOMING. ROLE CHANGE REQUESTED: ");
                if (Write_HCI_Command(message, hci_buf,10) == USB_SUCCESS)
                {
                    HciState = HCI_ROLE_CHANGE_WAIT;
                    DemoState = BT_STATE_READ_DATA;
                };
                break;

            case HCI_ROLE_CHANGE_WAIT:
                USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
                break;

            case HCI_CMD_CONNECTION_ACCEPTED:
                HciState = HCI_CMD_SCAN_DISABLE;
                break;

            case HCI_CMD_SCAN_DISABLE: //wait for controller to initiate L2CAP Connection Request
                hci_buf[0]=0x1a;
                hci_buf[1]=0x0c;
                hci_buf[2]=0x01;
                hci_buf[3]=0x00; //disable scan
                strcpy(message,"HCI_CMD_SCAN_DISABLE: ");
                if (Write_HCI_Command(message, hci_buf,4) == USB_SUCCESS)
                {
                    HciState = HCI_READ;
                };
                break;
//HCI_ACL ************************************************************************
//Respond to PS3 controller's request to open a connection channel for HID_Control 0x0011
            case L2CAP_CONNECT_RESP11:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x10;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0c;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x03;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x08;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_host_ctrl[0];
                acl_buf[13]=cid_host_ctrl[1];
                acl_buf[14]=cid_dev_ctrl[0];
                acl_buf[15]=cid_dev_ctrl[1];
                acl_buf[16]=0x00;
                acl_buf[17]=0x00;
                acl_buf[18]=0x00;
                acl_buf[19]=0x00;

                UART2PrintString("SCID CTRL = ");
                UART2PutHex(cid_host_ctrl[0]);
                UART2PrintString("\n\r");
                strcpy(message,"L2CAP_CONNECT_RESP11: ");
                if (Write_ACL_Command(message, acl_buf, 20) == USB_SUCCESS)
                {
                    HciState = L2CAP_CONFIG_REQ11; //No response from controller, go to config request
                }
                break;

            //Config request for Ch 0x0011
            case L2CAP_CONFIG_REQ11: //config request
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x10;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0c;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x04;
                acl_buf[9]=0x01;
                acl_buf[10]=0x08;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_ctrl[0];
                acl_buf[13]=cid_dev_ctrl[1];
                acl_buf[14]=0x00;
                acl_buf[15]=0x00;
                acl_buf[16]=0x01;
                acl_buf[17]=0x02;
                acl_buf[18]=0x40;
                acl_buf[19]=0x00;

                strcpy(message,"L2CAP_CONFIG_REQ11: ");
                if (Write_ACL_Command(message, acl_buf, 20) == USB_SUCCESS)
                {
                    HciState = HCI_READ; //Wait for response
                }
                break;

            //Respond to config request for Ch 0x0011
            case L2CAP_CONFIG_RESP11:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0e;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0a;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x05;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x06;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_ctrl[0];
                acl_buf[13]=cid_dev_ctrl[1];
                acl_buf[14]=0x00;
                acl_buf[15]=0x00;
                acl_buf[16]=0x00;
                acl_buf[17]=0x00;

                strcpy(message,"L2CAP_CONFIG_RESP11: ");
                if (Write_ACL_Command(message, acl_buf, 18) == USB_SUCCESS)
                {
                    HciState = HCI_READ; //wait for response
                }
                break;

            //Respond to PS3 controller's request to open a connection channel for HID_Interrupt 0x0013
            case L2CAP_CONNECT_RESP13:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x10;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0c;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x03;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x08;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_host_data[0];
                acl_buf[13]=cid_host_data[1];
                acl_buf[14]=cid_dev_data[0];
                acl_buf[15]=cid_dev_data[1];
                acl_buf[16]=0x00;
                acl_buf[17]=0x00;
                acl_buf[18]=0x00;
                acl_buf[19]=0x00;

                UART2PrintString("SCID DATA = ");
                UART2PutHex(cid_host_data[0]);
                UART2PrintString("\n\r");
                strcpy(message,"L2CAP_CONNECT_RESP13: ");
                if (Write_ACL_Command(message, acl_buf, 20) == USB_SUCCESS)
                {
                    HciState = L2CAP_CONFIG_REQ13;
                }
                break;

            case L2CAP_CONFIG_REQ13:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x10;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0c;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x04;
                acl_buf[9]=0x02;
                acl_buf[10]=0x08;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_data[0];
                acl_buf[13]=cid_dev_data[1];
                acl_buf[14]=0x00;
                acl_buf[15]=0x00;
                acl_buf[16]=0x01;
                acl_buf[17]=0x02;
                acl_buf[18]=0x40;
                acl_buf[19]=0x00;

                strcpy(message,"L2CAP_CONFIG_REQ13: ");
                if (Write_ACL_Command(message, acl_buf, 20) == USB_SUCCESS)
                {
                    HciState = HCI_READ; //wait for response
                }
                break;

            case L2CAP_CONFIG_RESP13:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0e;
                acl_buf[3]=0x00;
                acl_buf[4]=0x0a;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x05;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x06;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_data[0];
                acl_buf[13]=cid_dev_data[1];
                acl_buf[14]=0x00;
                acl_buf[15]=0x00;
                acl_buf[16]=0x00;
                acl_buf[17]=0x00;

                strcpy(message,"L2CAP_CONNECT_RESP13: ");
                if (Write_ACL_Command(message, acl_buf, 18) == USB_SUCCESS)
                {
                    HciState = PS3_BT_SET_FEATURE_INIT;
                    DemoState = BT_STATE_READ_DATA;
                }
                break;

            case PS3_BT_SET_FEATURE_INIT:
                //initialize outputReportBT fields for later use
                outputReportBT[0]=handle[0];
                outputReportBT[1]=handle[1]+0x20;
                outputReportBT[2]=0x36;
                outputReportBT[3]=0x00;
                outputReportBT[4]=0x32;
                outputReportBT[5]=0x00;
                outputReportBT[6]=cid_dev_ctrl[0];
                outputReportBT[7]=cid_dev_ctrl[1];

                InitHIDOutputReport();

                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0a;
                acl_buf[3]=0x00;
                acl_buf[4]=0x06;
                acl_buf[5]=0x00;
                acl_buf[6]=cid_dev_ctrl[0];
                acl_buf[7]=cid_dev_ctrl[1];
                acl_buf[8]=0x53;
                acl_buf[9]=0xf4;
                acl_buf[10]=0x42;
                acl_buf[11]=0x03;
                acl_buf[12]=0x00;
                acl_buf[13]=0x00;

                strcpy(message,"PS3_BT_SET_FEATURE_INIT: ");
                if (Write_ACL_Command(message, acl_buf, 14) == USB_SUCCESS)
                {
                    HciState = PS3_BT_RUNNING;
                }
                break;

            case PS3_BT_RUNNING:
                USBHostGenericAclRead(deviceAddress, acl_buf, DATA_PACKET_LENGTH);
                USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
                break;

             case L2CAP_DISCONNECT_DATA_RESP:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0c;
                acl_buf[3]=0x00;
                acl_buf[4]=0x08;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x07;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x04;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_host_data[0];
                acl_buf[13]=cid_host_data[1];
                acl_buf[14]=cid_dev_data[0];
                acl_buf[15]=cid_dev_data[1];
                strcpy(message,"L2CAP_DISCONNECT_DATA_RESP: ");
                if (Write_ACL_Command(message, acl_buf, 16) == USB_SUCCESS)
                {
                    HciState = PS3_BT_RUNNING;
                }
                break;

             case L2CAP_DISCONNECT_CTRL_RESP:
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0c;
                acl_buf[3]=0x00;
                acl_buf[4]=0x08;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x07;
                acl_buf[9]=packet_id;
                acl_buf[10]=0x04;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_host_ctrl[0];
                acl_buf[13]=cid_host_ctrl[1];
                acl_buf[14]=cid_dev_ctrl[0];
                acl_buf[15]=cid_dev_ctrl[1];
                strcpy(message,"L2CAP_DISCONNECT_CTRL_RESP: ");
                if (Write_ACL_Command(message, acl_buf, 16) == USB_SUCCESS)
                {
                    HciState = PS3_BT_RUNNING;
                }
                break;

            case L2CAP_DISCONNECT_DATA:
                packet_id++;
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0c;
                acl_buf[3]=0x00;
                acl_buf[4]=0x08;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x06; //disconnect command
                acl_buf[9]=packet_id;
                acl_buf[10]=0x04;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_data[0];
                acl_buf[13]=cid_dev_data[1];
                acl_buf[14]=cid_host_data[0];
                acl_buf[15]=cid_host_data[1];
                strcpy(message,"L2CAP_DISCONNECT_DATA: ");
                if (Write_ACL_Command(message, acl_buf, 16) == USB_SUCCESS)
                {
                    HciState = HCI_READ;
                    DemoState = BT_STATE_READ_DATA;
                }
                break;

            case L2CAP_DISCONNECT_CTRL:
                packet_id++;
                acl_buf[0]=handle[0];
                acl_buf[1]=handle[1]+0x20;
                acl_buf[2]=0x0c;
                acl_buf[3]=0x00;
                acl_buf[4]=0x08;
                acl_buf[5]=0x00;
                acl_buf[6]=0x01;
                acl_buf[7]=0x00;
                acl_buf[8]=0x06; //disconnect command
                acl_buf[9]=packet_id;
                acl_buf[10]=0x04;
                acl_buf[11]=0x00;
                acl_buf[12]=cid_dev_ctrl[0];
                acl_buf[13]=cid_dev_ctrl[1];
                acl_buf[14]=cid_host_ctrl[0];
                acl_buf[15]=cid_host_ctrl[1];
                strcpy(message,"L2CAP_DISCONNECT_DATA: ");
                if (Write_ACL_Command(message, acl_buf, 16) == USB_SUCCESS)
                {
                    HciState = HCI_READ;
                    DemoState = BT_STATE_READ_DATA;
                }
                break;

            case HCI_DISCONNECT:
                hci_buf[0]=0x06;
                hci_buf[1]=0x01 << 2;
                hci_buf[2]=0x03;
                hci_buf[3]=handle[0];
                hci_buf[4]=handle[1];
                hci_buf[5]=0x13; //user terminated connection

                strcpy(message,"HCI_DISCONNECT: ");
                if (Write_HCI_Command(message, hci_buf,6)== USB_SUCCESS)
                {
                    ResetStates();
                }
                break;

            case HCI_READ:
                USBHostGenericAclRead(deviceAddress, acl_buf, DATA_PACKET_LENGTH);
                USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
                break;
        }
        break;

    case BT_STATE_READ_DATA:
        USBHostGenericAclRead(deviceAddress, acl_buf, DATA_PACKET_LENGTH);
        USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
        break;

    default:
        UART2PrintString("Unknown Demo State.\n\r");
        ResetStates();
        break;
    }
    DelayMs(1); // 1ms delay.  Not sure why we need this, but seems to prevent certain errors.

} // ManageDemoStateBluetooth

void ManageDemoStateWire ( void )
{
    BYTE usbReturnVal;

    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
	#ifdef DEBUG_MODE
        UART2PrintString( "Controller detached - polled\r\n" );
	#endif
        ResetStates();
        SetConnectedDeviceType(NONE);
    }
    switch (PS3WireState){
        case PS3_WIRE_WRITE_BD_ADDR:
            UART2PrintString( "Pair Controller with last Bluetooth dongle...\n\r" );
            localBdAddr.ints[0] = DataEERead(EEPROM_BT_ADDR_0);
            localBdAddr.ints[1] = DataEERead(EEPROM_BT_ADDR_1);
            localBdAddr.ints[2] = DataEERead(EEPROM_BT_ADDR_2);

            hci_buf[0] = 0x01;
            hci_buf[1] = 0x00;
            hci_buf[2] = localBdAddr.bytes[0];
            hci_buf[3] = localBdAddr.bytes[1];
            hci_buf[4] = localBdAddr.bytes[2];
            hci_buf[5] = localBdAddr.bytes[3];
            hci_buf[6] = localBdAddr.bytes[4];
            hci_buf[7] = localBdAddr.bytes[5];

            if (USB_SUCCESS == USBHostIssueDeviceRequest( deviceAddress, 0x21, 0x09, 0x03f5, 0, 8, hci_buf, USB_DEVICE_REQUEST_SET, 1))
            {
                UART2PrintString( "Done.\n\r" );
                PS3WireState = PS3_WIRE_READ_BD_ADDR;
            }
            else
            {
                UART2PrintString( "Device Write Error !\r\n" );
            }
            for (data_num = 2; data_num < 8; data_num++)
            {
                UART2PutHex(hci_buf[data_num]);
                UART2PutChar(' ');
            }
            UART2PrintString("\n\r");
            break;

        case PS3_WIRE_READ_BD_ADDR:
            UART2PrintString( "Read Back ADDR.\n\r" );
            if (USB_SUCCESS == USBHostIssueDeviceRequest( deviceAddress, 0xA1, 0x01, 0x03f5, 0, 8, hci_buf, USB_DEVICE_REQUEST_GET, 1)){
                PS3WireState = PS3_WIRE_CONTROLLER_INIT;
                for (data_num = 2; data_num < 8; data_num++)
                {
                    UART2PutHex(hci_buf[data_num]);
                    UART2PutChar(' ');
                }
                UART2PrintString("\n\r");
            }
            break;

        case PS3_WIRE_CONTROLLER_INIT:
            UART2PrintString( "Initialize Controller.\n\r" );
            if (USB_SUCCESS == USBHostIssueDeviceRequest( deviceAddress, 0x21, 0x09, 0x03f4, 0, 4, PS3WireInitialize, USB_DEVICE_REQUEST_SET, 1))
            {
                PS3WireState = PS3_WIRE_RUNNING;
            }
            else
            {
                UART2PrintString( "Device Write Error !\r\n" );
            }
            break;

        case PS3_WIRE_RUNNING:
            usbReturnVal=USBHostGenericRead(deviceAddress, hci_buf, DATA_PACKET_LENGTH);
            //UART2PrintString("Running\n\r");

            if ( usbReturnVal != USB_SUCCESS )
            {
                UART2PutHex(usbReturnVal);
                UART2PrintString( " - Device Read Error !\r\n" );
            }
            break;

        default:
            break;
    }


    DelayMs(1); // 1ms delay
} // ManageDemoSt


//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *S
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns TRUE.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch (event)
    {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof(GENERIC_DEVICE_ID))
            {
                deviceAddress   = ((GENERIC_DEVICE_ID *)data)->deviceAddress;
                DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET;//YTS !!!!!!!!!!!!!!!!!
                PS3WireState = PS3_WIRE_WRITE_BD_ADDR; //MH

                UART2PrintString( "Generic demo device attached - event, deviceAddress=" );
                UART2PutDec( deviceAddress );
                UART2PrintString( "\r\n" );
                #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((GENERIC_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        UART2PutChar( ((GENERIC_DEVICE_ID *)data)->serialNumber[i] );
                    }
                #endif
                UART2PrintString( "\r\n" );
                return TRUE;
            }
            break;

        case EVENT_GENERIC_DETACH:
            UART2PrintString( "Generic demo device detached - event\r\n" );
            ResetStates();
            SetConnectedDeviceType(NONE);
            return TRUE;
            break;

        case EVENT_GENERIC_TX_DONE:           // The main state machine will poll the driver.
            break;
        case EVENT_GENERIC_RX1_DONE://YTS
            if (USBHostGenericGetRx1Length(deviceAddress) > 0){
                switch (GetConnectedDeviceType() )
                {
                    case BLUETOOTH:
                        ParseRx1PS3BT();
                        break;
                    case CONTROLLER:
                        ParseRx1PS3Wire();
                        break;
                    default:
                        break;
                }
            }
            return TRUE;
            break;

        case EVENT_GENERIC_RX2_DONE://YTS
            if (USBHostGenericGetRx2Length(deviceAddress) > 0)
            {
                switch (GetConnectedDeviceType() )
                {
                case BLUETOOTH:
                    ParseRx2PS3BT();
                    break;
                case CONTROLLER:
                    break;
                default:
                    break;
                }
            }
            return TRUE;
            break;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;
            break;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;
            break;

        case EVENT_HUB_ATTACH:
            UART2PrintString( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }
    return FALSE;
} // USB_ApplicationEventHandler



//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    unsigned char uartIn;
    unsigned char charbuf[2] = {0};
    
     // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nServoShock Firmware Rev ");
        UART2PrintString(FIRMWARE_REV);
        UART2PrintString(": Could not initialize system.  Halting.\r\n\r\n" );
        while (1);
    }
    if ( USBHostInit(0) == TRUE )
    {
        UART2PrintString( "\r\n\r\n***** ServoShock Firmware Rev ");
        UART2PrintString(FIRMWARE_REV);
        UART2PrintString(" *****\r\n\r\n" );
    }
    else
    {
        UART2PrintString( "\r\n\r\nServoShock Firmware Rev ");
        UART2PrintString(FIRMWARE_REV);
        UART2PrintString(" Could not initialize USB.  Halting.\r\n\r\n" );
        while (1);
    }

    //Enter test mode
    if (_PGD == 1)
    {
        UART2PrintString("Circuit Test Mode.\n\r");
        while(1){
            TestCases();
        }
    }

    //Initialize non-volatile memory
    DataEEInit();
    dataEEFlags.val = 0;

    if (DataEERead(DEFAULTS_INITIALIZED_FLAG) != VERSION_CODE)
    {
        LoadDefaultSettings(TRUE,0);

        if (DataEEWrite(VERSION_CODE, DEFAULTS_INITIALIZED_FLAG) == 0){
            UART2PrintString("Defaults Loaded.\n\r");
            SaveSettings();
        }
        else
        {
           UART2PrintString("EEPROM Write Error. ");
           UART2PrintString("\n\rErasing page.\n\r");
           ErasePage(0,0);
           ErasePage(0,1);
           Reset();
            
        }
    }
    else
    {
        LoadSavedSettings();
        UART2PrintString("Firmware Code: ");
        UART2PutDecInt(DataEERead(DEFAULTS_INITIALIZED_FLAG));
        UART2PrintString("\n\rLoading Saved Settings. \n\r");
    }

    ResetOutputs(ControllerInPtr);  //initialize outputs and put in an artificial nominal input so it doesn't read all zeros.


    //enable timer4 interrupt
    IEC1bits.T4IE = 1;
    IPC6bits.T4IP = 1;//lowest priority interrupt
    
    //SPI setup
    initSPI();
    IEC1bits.INT1IE = 1;

    InitOutputs(); //enable outputs

    // Main Processing Loop
    while (1)
    {
        //INDICATOR_LED = 0;
         UART2ClrError();
        if (UART2DataReceived())
        {
            uartIn = UART2GetChar();
            //UART2PutChar(uartIn);               
            if (charbuf[1] == 'R' && charbuf[0] == 'S' && uartIn == 'T') Reset();
            charbuf[1] = charbuf[0];
            charbuf[0] = uartIn;

        }


#ifdef DEBUG_MODE
        if (LED_SEL_0 == 1)
        {
            UART2PutHex(DemoState);
            UART2PutChar('-');
            UART2PutHex(HciState);
            UART2PutChar(' ');
        }
#endif
   
        // Maintain USB Host State
        USBHostTasks();
        if (GetConnectedDeviceType() == CONTROLLER)
        {
            ManageDemoStateWire();
        }
        else if (GetConnectedDeviceType() == BLUETOOTH)
        {
            ManageDemoStateBluetooth();
        }
        else
        {
        }


        //Process buttons here
        if (HciState == PS3_BT_RUNNING || PS3WireState == PS3_WIRE_RUNNING)
        {
            INDICATOR_LED = 0;
            //Process Device Type specific tasks
            switch (GetConnectedDeviceType())
            {
                case BLUETOOTH:
                    if (GetPressDuration()->start > 300 && GetPressDuration()->select > 300)
                    {
                        HciState = L2CAP_DISCONNECT_DATA;
                    }

                    if ( (GetTimeoutSetting() > 0) && (GetIdleTimer() > GetTimeoutSetting()) )
                    {
                        UART2PrintString("Idle Timeout.\n\r");
                        HciState = L2CAP_DISCONNECT_DATA;
                    }

                    break;

                case CONTROLLER:
                    break;

                default:
                    break;
            }

            if (getSpiInputPacket()->overrideLED) //Control LED with SPI
            {
                LEDVal = getSpiInputPacket()->LEDValue;
            }
            else //Control LED with I/O. Only enough combinations for 4 light patterns
            {
                LEDVal = 0x01<<(LED_SEL_1*2+LED_SEL_0);
            }

            if (AD1CON1bits.DONE) //Control Vibe with ADC
            {
            }

            if (getSpiInputPacket()->overrideRumbleL)
            {
                vibeL = getSpiInputPacket()->RumbleL;

            }
            else if (AD1CON1bits.DONE)
            {
                vibeL = (unsigned char)(ADC1BUF0>>2); //convert 10-bit to 8-bit
            }
            
            if (getSpiInputPacket()->overrideRumbleH)
            {
                vibeH = getSpiInputPacket()->RumbleH;
            }
            else if (AD1CON1bits.DONE) //Control Vibe with SPI bus
            {
                vibeH = (unsigned char)(ADC1BUF1>>2);
            }

            ////////////100ms loop for writing LEDs and rumble motor values to controller////////////////
            if (loopCounter >= 9) //time the outputs to avoid spamming the controller.  Spamming will cause it to lag or freeze//
            {
                INDICATOR_LED = 1;
                loopCounter = 0;
                //update the feedback duration countdown timer
                UpdateRumbleFeedback(0);

                //Process Device Type specific timed tasks
                switch (GetConnectedDeviceType())
                {
                    case BLUETOOTH:
                        UpdateBatteryDisplay(ControllerInPtr);  //display battery levels on LEDs if the command was activated
                        break;

                    case CONTROLLER:
                        break;

                    case NONE:
                        break;

                    default:
                        break;
                }

                //Write OUTPUT
                strcpy(message,"WRITE_STATE: ");
                
                if (WriteOutputReport() == USB_SUCCESS)
                {
                    //Do something to indicate success?
                #ifdef DEBUG_MODE
                    UART2PrintString("OK:");
                #endif
                }
                
            }
        }
        //Stuff to do when the controllers are not connected
        //exit the config state if we were left in that state when the controller was connected
        else
        {
            INDICATOR_LED = 0;
            ResetIdleTimer();  //need to reset the idle timer because being disconnected makes it appear idle and can trigger the auto-disconnect.
            GetPS3Report()[0] = 0xFF;  //set first byte to 0xFF so anything reading the packs from SPI bus can tell it's disconnected
            if (configFlag == TRUE)
            {
                configFlag = FALSE;
                ConfigOutput(configFlag);                
            }
            if (loopCounter == 150)
            {
                INDICATOR_LED = 1;
            }
            if (loopCounter > 160)
            {
                loopCounter = 0;
            }
        }
        DelayMs(1);
    }
    return 0;

} // main

//10ms interrupt to poll/update states
void __attribute__((__interrupt__, auto_psv))_T4Interrupt(void)
{
    //INDICATOR_LED = 1;
    loopCounter++; //counts how many times we enter this ISR routine.
    SpiInputOverride(); // Check to see if we're overriding any inputs with SPI bus.
    UpdateNewPress(); //Update record of button presses
        //enter or exit config mode here
    if (GetPressDuration()->select > 300 && GetPressDuration()->psButton > 300){
        configFlag = TRUE;
    }
    if (configFlag == TRUE)
    {
        configFlag = ConfigOutput(configFlag);  //configFlag will be set to false when we want to exit config, and breaks this loop.
    }
    else
    {
        UpdateButtonOutputs(FALSE, 0); //don't update button outputs when in config mode because buttons are used to change settings.
        SetRumbleL(vibeL);
        SetRumbleH(vibeH);
        SetLEDBin(LEDVal);
    }
    UpdateServoOutputs();

    AD1CON1bits.SAMP = 1; //sample ADC
    IFS1bits.T4IF = 0;
}

//SPI bus interrupt to handle SPI packets
void __attribute__((__interrupt__, auto_psv))_SPI1Interrupt(void)
{
    while(!SPI1STATbits.SPITBF)
    {
        if (spi_index < PS3_REPORT_LENGTH)
        {
            if (spi_index < SPI_INPUT_PACKET_LENGTH)
            {
                getSpiInputPacket()->array[spi_index] = SPI1BUF;
            }
            else
            {
                temp = SPI1BUF; //burn off filler bytes
            }
            SPI1BUF = GetPS3Report()[spi_index]; //fill output buffer
        }

        //tack on data about the LED and rumble states onto the end of the packet
        else if (spi_index == PS3_REPORT_LENGTH)
        {
            SPI1BUF = GetOutputPacket()->rumbleLPower;
            temp = SPI1BUF; //burn off filler bytes
        }
        else if (spi_index == (PS3_REPORT_LENGTH + 1))
        {
            SPI1BUF = GetOutputPacket()->rumbleHDuration*43;
            temp = SPI1BUF; //burn off filler bytes
        }
        else if (spi_index == (PS3_REPORT_LENGTH + 2))
        {
            SPI1BUF = ((GetOutputPacket()->LED)>>1 & 0b00001111); //bit mask for the 4 LEDs
            temp = SPI1BUF;
        }
        else
        {
            SPI1BUF = 0xAA; //padding
            temp = SPI1BUF; //burn off bytes
        }
        spi_index++;       
    }
    IFS0bits.SPI1IF = 0;
}

//INT1Interrupt: The INT1Interrupt pin (pin 46) is tied to the SPI slave select pin (pin 45).
//execute when SPI slave select is pulled high to clear/prime the input/output buffers, so the messages will always be the same length
void __attribute__((__interrupt__, auto_psv))_INT1Interrupt(void) 
{
    if(SPI1STATbits.SPIEN) //if we try to run the code in the interrupt when SPI bus is disabled, it will get stuck!
    {
        while(!SPI1STATbits.SPITBF)
        {
            //The PIC can't manually flush the 8-byte output buffer, so we'll  fill it up so that we
            //start the packet on the 9th byte every time.  Discard the first 8 byes
            //on the receiving end.
            SPI1BUF = 0xAB;
        }
        
        while(!SPI1STATbits.SRXMPT)
        {
            temp = SPI1BUF; //fush the input buffer.
        }
        spi_index = 0;
        SPI1STATbits.SPIROV = 0;
        IFS1bits.INT1IF = 0;
    }
}


//Error Traps
void __attribute__((__interrupt__, auto_psv))_AddressError(void)
{
    UART2PutChar('E');
    {__asm__ volatile ("nop");}
}

void __attribute__((__interrupt__, auto_psv))_StackError(void)
{
    UART2PrintString("StackError");
    {__asm__ volatile ("retfie");}
}

void __attribute__((__interrupt__, auto_psv))_MathError(void)
{
    UART2PrintString("Math Error");
    {__asm__ volatile ("retfie");}
}
/*************************************************************************
 * EOF main.c
 */

