/***********************************************************************
    PS3 USB Host
    Copyright (C) 2013 Cross Product Creations

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************/

//This file holdes the functions for configuring and setting the outputs.

#ifndef OUTPUTS_H
#define	OUTPUTS_H

#include <GenericTypeDefs.h>
#include <p24Fxxxx.h>
#include "PS3_controller.h"

#define AUTOCLICK_TIMER 2 //MUST be greater than 0, determines how fast the increment/decrement go when button is held
#define AUTOCLICK_DELAY 50 //dead time before autoclick kicks in
#define FAST_AUTOCLICK_DELAY 1000 //dead time before faster autoclick kicks in (for parameters with potentially large values)
#define FAST_AUTOCLICK_SIZE 10 //how many steps the fast increment takes at a time.

#define NUM_SERVOS 10
#define NUM_BUTTONS 17

//Servo parameter values.   When choosing values, beware of overflow if the values are multiplied by a scalar.
#define OFFSET_MAX 200
#define OFFSET_MIN -200
#define RANGE_MAX 250
#define RANGE_MIN 0
#define SENSITIVITY_MAX 200
#define SENSITIVITY_MIN 1
#define DEADBAND_MIN 0
#define DEADBAND_MAX 125
#define STOP_LOW 500 //hard stop low at pulse width 0.025 ms.  This is not a user configurable setting.
#define STOP_HIGH 5500 //hard stop high at pulse width 0.275 ms. This is not a user configurable setting.
#define TRIM_MAX 200
#define TRIM_MIN -200

#define AUTOFIRE_PERIOD_MAX 60000
#define AUTOFIRE_PERIOD_MIN 2
#define PULSE_WIDTH_MAX 60000
#define PULSE_WIDTH_MIN 1

/////////////////////
#define SERVO_PERIOD_MS_MAX 20 //20ms with a 2MHz counter
#define SERVO_PERIOD_MS_MIN 10

#define PWM_FREQ_MAX_KHZ 20
#define PWM_FREQ_MIN_KHZ 1

#define BUTTON_OUT_0 _LATF3
#define BUTTON_OUT_1 _LATC13
#define BUTTON_OUT_2 _LATC14
#define BUTTON_OUT_3 _LATD1
#define BUTTON_OUT_4 _LATD2
#define BUTTON_OUT_5 _LATD3
#define BUTTON_OUT_6 _LATD4
#define BUTTON_OUT_7 _LATD5
#define BUTTON_OUT_8 _LATD6
#define BUTTON_OUT_9 _LATD7
#define BUTTON_OUT_10 _LATF0
#define BUTTON_OUT_11 _LATF1
#define BUTTON_OUT_12 _LATE0
#define BUTTON_OUT_13 _LATE1
#define BUTTON_OUT_14 _LATE2
#define BUTTON_OUT_15 _LATE3
#define BUTTON_OUT_16 _LATE4
#define INDICATOR_LED _LATE5



////////////////////DATA TYPES//////////////////////////////////////////

//netbeans doesn't parse correctly if enum is buried in a struct.  Workaround by using typedef.
typedef enum {FORWARD,REVERSE} SERVO_DIR;
typedef enum {ABSOLUTE, RELATIVE} STICK_ABSOLUTE_RELATIVE;
typedef enum {SERVO, MOTOR} SERVO_MOTOR;
typedef enum {OFF, L_STICK, R_STICK} HOLD_RECALL;
#define SERVO_SETTINGS_LENGTH 13
typedef union _SERVO_SETTINGS
{
    struct __attribute__ ((aligned (2))) {
        signed int offset;
        signed int range;
        signed int deadband;
        signed int sensitivity;
        SERVO_DIR direction;
        STICK_ABSOLUTE_RELATIVE absOrRel;
        SERVO_MOTOR servoOrMotor;
        signed int servoPeriodMs;
        signed int pwmFrequency;
        signed int trim;
        signed int zeroPosition;
        HOLD_RECALL holdRecall;
        unsigned int servoRecallValue;
    };
    unsigned int array[SERVO_SETTINGS_LENGTH];
}SERVO_SETTINGS;

typedef enum {PUSHBUTTON, TOGGLE, SINGLE_SHOT, AUTOFIRE, TOGGLE_AUTOFIRE} BUTTON_OUTPUT_MODE; //netbeans doesn't parse correctly if enum is buried in a struct.  Workaround by using typedef.
typedef union _BUTTON_SETTINGS
{
    struct __attribute__ ((aligned (2))){
        BOOL invert;
        BUTTON_OUTPUT_MODE outputMode;
        unsigned int pulseWidth;
        unsigned int autofirePeriod;
    };
    unsigned int array[4];
}BUTTON_SETTINGS;



////////////////////FUNCTIONS//////////////////////////////////////////
void InitOutputs(void);
void LoadDefaultSettings(BOOL loadAll, unsigned int index);
void LoadSavedSettings(void);
void SaveSettings(void);
SERVO_SETTINGS* GetServoSettings(void);
BUTTON_SETTINGS* GetButtonSettings (void);
unsigned int GetTimeoutSetting(void);
BOOL ConfigOutput(BOOL configFlag);

void UpdateButtonOutputs(unsigned int configFlag, unsigned int configIndex);
void UpdateServoOutputs(void);
void ResetOutputs(CONTROLLER_IN* ControllerIn);
unsigned int GetSpiBusMode(void);
void PrintIndexLabel(unsigned int index);

/////////////PRIVATE FUNCTIONS/////////////////////////
void SaveLoadSubroutine(BOOL save_switch);

#endif

